@@ -35,6 +35,7 @@ from .ad_exceptions import ReplicationFailedException
 from .ad_exceptions import SamAccountNameNotUnique
 from .ad_exceptions import UserNotFoundException
 from .ad_logger import start_logging
+from .ad_template_engine import INVALID
 from .ad_template_engine import prepare_field_templates
 from .ad_template_engine import template_powershell
 from .user_names import UserNameGen
@@ -289,7 +290,7 @@ class MORESTSource(MODataSource):
         return mo_user
 
     def get_email_address(self, uuid):
-        mail_dict = first(self.helper.get_e_addresses(uuid, scope="EMAIL"))
+        mail_dict = first(self.helper.get_e_addresses(uuid, scope="EMAIL"), default={})
         return dict_subset(mail_dict, ["uuid", "value"])
 
     def find_primary_engagement(self, uuid):
@@ -358,6 +359,12 @@ class MORESTSource(MODataSource):
 
 
 class ADWriter(AD):
+    INVALID_UNIT_ADDRESS = {
+        "city": INVALID,
+        "postal_code": INVALID,
+        "streetname": INVALID,
+    }
+
     def __init__(self, lc=None, lc_historic=None, **kwargs):
         super().__init__(**kwargs)
         self.settings = self.all_settings
@@ -554,23 +561,25 @@ class ADWriter(AD):
             return None
 
         def split_addresses(addresses):
-            postal_code = city = streetname = "Ukendt"
-            if addresses.get("postal"):
-                postal = addresses["postal"]
+            try:
+                postal = addresses["postal"]["Adresse"]
+            except KeyError:
+                return ADWriter.INVALID_UNIT_ADDRESS
+            else:
                 try:
-                    postal_code = re.findall("[0-9]{4}", postal["Adresse"])[0]
-                    city_pos = postal["Adresse"].find(postal_code) + 5
-                    city = postal["Adresse"][city_pos:]
-                    streetname = postal["Adresse"][: city_pos - 7]
-                except IndexError:
+                    postal_code = re.findall("[0-9]{4}", postal)[0]
+                    city_pos = postal.find(postal_code) + 5
+                    city = postal[city_pos:]
+                    streetname = postal[: city_pos - 7]
+                except (IndexError, TypeError):
                     logger.error("Unable to read adresse from MO (no access to DAR?)")
-                except TypeError:
-                    logger.error("Unable to read adresse from MO (no access to DAR?)")
-            return {
-                "postal_code": postal_code,
-                "city": city,
-                "streetname": streetname,
-            }
+                    return ADWriter.INVALID_UNIT_ADDRESS
+                else:
+                    return {
+                        "postal_code": postal_code,
+                        "city": city,
+                        "streetname": streetname,
+                    }
 
         def read_manager_uuid(mo_user, eng_uuid):
             manager_uuid = self.datasource.get_manager_uuid(mo_user, eng_uuid)
@@ -750,15 +759,9 @@ class ADWriter(AD):
         return ps_script
 
     def _compare_fields(self, ad_field, value, ad_user):
-        logger.info("Check AD field: {}".format(ad_field))
         mismatch = {}
         ad_field_value = ad_user.get(ad_field)
 
-        if value is None:
-            msg = "Value for {} is None-type replace to string None"
-            logger.debug(msg.format(ad_field))
-            value = "None"
-
         # Some AD fields contain one-element lists, rather than the usual strings,
         # numbers or UUIDs.
         # In such cases, we "unpack" the single-element list before comparing it to the
@@ -766,10 +769,28 @@ class ADWriter(AD):
         if isinstance(ad_field_value, list) and len(ad_field_value) == 1:
             ad_field_value = ad_field_value[0]
 
-        if ad_field_value != value:
+        # The "MO value" in `value` actually comes from a call to
+        # `_render_field_template`, which may produce the string `"None"` when it
+        # encounters a `None`.
+        if value == "None":
+            value = None
+
+        # We also consider the reverse situation where the AD field contains the string
+        # "None" to indicate an empty value.
+        if ad_field_value == "None":
+            ad_field_value = None
+
+        # Do the actual comparison
+        if value == INVALID:
+            msg = "%r: MO value is INVALID, not changing AD value %r"
+            logger.info(msg, ad_field, ad_field_value)
+        elif ad_field_value != value:
             msg = "%r: AD value %r does not match MO value %r"
             logger.info(msg, ad_field, ad_field_value, value)
             mismatch = {ad_field: (ad_field_value, value)}
+        else:
+            msg = "%r: AD value %r already matches MO value %r"
+            logger.debug(msg, ad_field, ad_field_value, value)
 
         return mismatch
 
@@ -884,13 +905,22 @@ class ADWriter(AD):
             mismatch.update(self._compare_fields(ad_field, rendered_value, ad_user))
 
         if mo_values.get("manager_cpr"):
-            manager_ad_user = self._find_ad_user(
-                mo_values["manager_cpr"], ad_dump=ad_dump
-            )
-            manager_distinguished_name = manager_ad_user["DistinguishedName"]
-            if ad_user["manager"] != manager_distinguished_name:
-                mismatch["manager"] = (ad_user["manager"], manager_distinguished_name)
-                logger.info("Manager should be updated")
+            try:
+                manager_ad_user = self._find_ad_user(
+                    mo_values["manager_cpr"], ad_dump=ad_dump
+                )
+            except CprNotFoundInADException:
+                logger.warning(
+                    "could not find manager by cpr %r", mo_values.get("manager_cpr")
+                )
+            else:
+                manager_distinguished_name = manager_ad_user["DistinguishedName"]
+                if ad_user["manager"] != manager_distinguished_name:
+                    mismatch["manager"] = (
+                        ad_user["manager"],
+                        manager_distinguished_name,
+                    )
+                    logger.info("Manager should be updated")
 
         return mismatch
 
