@@ -38,6 +38,9 @@ class MockADParameterReader(TestADWriterMixin):
     def cache_all(self, **kwargs):
         return self.read_it_all()
 
+    def get_all_samaccountname_values(self):
+        return {self.read_user()["SamAccountName"]}
+
 
 class MockEmptyADReader(MockADParameterReader):
     """Mock implementation of `ADParameterReader` which simulates an empty AD"""
@@ -111,10 +114,15 @@ class MockLoraCache:
                     {
                         "uuid": eng["uuid"],
                         "user": self._mo_values["uuid"],
+                        "unit": MO_ROOT_ORG_UNIT_UUID,
+                        "user_key": "engagement_%s" % self._mo_values["uuid"],
                         "primary_boolean": eng["is_primary"],
                         "from_date": eng["validity"]["from"],
                         "to_date": eng["validity"]["to"],
                         "extensions": extensions,
+                        "job_function": eng.get("job_function", {}).get("uuid"),
+                        "primary_type": None,
+                        "engagement_type": None,
                     }
                 ]
                 for eng in self._mo_engagements
@@ -125,6 +133,8 @@ class MockLoraCache:
                     {
                         "uuid": "engagement_uuid",
                         "user": self._mo_values["uuid"],
+                        "unit": MO_ROOT_ORG_UNIT_UUID,
+                        "user_key": "engagement_%s" % self._mo_values["uuid"],
                         "primary_boolean": True,
                         "from_date": "1960-01-01",
                         "to_date": None,
@@ -249,9 +259,42 @@ class MockLoraCacheParentUnitUnset(MockLoraCacheExtended):
         }
 
 
+class MockLoraCacheUnitAddress(MockLoraCacheExtended):
+
+    _job_function_class_uuid = "job_function_class_uuid"
+
+    def __init__(self, address_value):
+        self._address_value = address_value
+        mo_values = {"uuid": MO_UUID}
+        mo_engagements = [
+            {
+                "uuid": "eng_%s" % MO_UUID,
+                "is_primary": True,
+                "validity": {"from": "1960-01-01", "to": None},
+                "job_function": {"uuid": self._job_function_class_uuid},
+            }
+        ]
+        super().__init__(mo_values, mo_engagements=mo_engagements)
+
+    @property
+    def classes(self):
+        return {self._job_function_class_uuid: {"title": "Job function title"}}
+
+    @property
+    def addresses(self):
+        address_uuid = "address-uuid"
+        address = {
+            "unit": MO_ROOT_ORG_UNIT_UUID,
+            "scope": "DAR",
+            "value": self._address_value,
+        }
+        return {address_uuid: [address]}
+
+
 class MockMoraHelper(MoraHelper):
-    def __init__(self, cpr):
+    def __init__(self, cpr, read_ou_addresses=None):
         self._mo_user = {"cpr_no": cpr, "uuid": MO_UUID}
+        self._read_ou_addresses = read_ou_addresses or {}
         self._read_user_calls = []
         super().__init__()
 
@@ -276,6 +319,12 @@ class MockMoraHelper(MoraHelper):
             "parent": None,
         }
 
+    def read_ou_address(self, uuid, scope=None, **kwargs):
+        if scope == "EMAIL":
+            return []
+        if scope == "DAR":
+            return self._read_ou_addresses
+
     def read_engagement_manager(self, engagement_uuid):
         return {}
 
@@ -323,6 +372,7 @@ class MockADWriterContext(ExitStack):
         super().__init__()
         settings = copy.deepcopy(self.all_settings)
         template_to_ad_fields = kwargs.get("template_to_ad_fields", {})
+        self._read_ou_addresses = kwargs.get("read_ou_addresses")
         settings["primary_write"]["template_to_ad_fields"].update(template_to_ad_fields)
         self._settings = settings
 
@@ -347,5 +397,9 @@ class MockADWriterContext(ExitStack):
             return_value=MockADParameterReader(),
         )
         yield patch(
-            f"{prefix}.ad_writer.MoraHelper", return_value=MockMoraHelper(cpr="")
+            f"{prefix}.ad_writer.MoraHelper",
+            return_value=MockMoraHelper(
+                cpr="",
+                read_ou_addresses=self._read_ou_addresses,
+            ),
         )
