@@ -5,6 +5,7 @@ from unittest import TestCase
 from freezegun import freeze_time
 from jinja2.exceptions import UndefinedError
 from more_itertools import first_true
+from more_itertools import only
 from os2mo_helpers.mora_helpers import MoraHelper
 from parameterized import parameterized
 from ra_utils.lazy_dict import LazyDict
@@ -14,12 +15,14 @@ from ..ad_exceptions import CprNotNotUnique
 from ..ad_exceptions import NoPrimaryEngagementException
 from ..ad_exceptions import SamAccountNameNotUnique
 from ..ad_template_engine import illegal_parameters
+from ..ad_template_engine import INVALID
 from ..ad_writer import ADWriter
 from ..ad_writer import LoraCacheSource
 from ..user_names import UserNameSetInAD
 from ..utils import AttrDict
 from .mocks import MO_UUID
 from .mocks import MockADWriterContext
+from .mocks import MockLoraCacheUnitAddress
 from .mocks import MockMORESTSource
 from .test_utils import dict_modifier
 from .test_utils import mo_modifier
@@ -1084,7 +1087,11 @@ class TestADWriter(TestCase, TestADWriterMixin):
 class _TestRealADWriter(TestCase):
     def _prepare_adwriter(self, **kwargs):
         template_to_ad_fields = kwargs.pop("template_to_ad_fields", {})
-        with MockADWriterContext(template_to_ad_fields=template_to_ad_fields):
+        read_ou_addresses = kwargs.pop("read_ou_addresses", None)
+        with MockADWriterContext(
+            template_to_ad_fields=template_to_ad_fields,
+            read_ou_addresses=read_ou_addresses,
+        ):
             instance = ADWriter(**kwargs)
             instance.get_from_ad = lambda *_args, **_kwargs: {}
             return instance
@@ -1126,6 +1133,12 @@ class TestInitNameCreator(_TestRealADWriter):
 
 
 class TestSyncCompare(_TestRealADWriter):
+    _ad_user_employee = {
+        "cpr_field": "cpr",
+        "SamAccountName": "sam",
+        "Manager": "old_manager_dn",
+    }
+
     def test_compare_fields_converts_ad_list(self):
         """Some AD fields contain one-element lists, rather than the usual strings,
         numbers or UUIDs.
@@ -1146,6 +1159,133 @@ class TestSyncCompare(_TestRealADWriter):
         self.assertIn("ad_field_name", mismatch)
         self.assertEqual(mismatch["ad_field_name"], (ad_list_element, mo_value))
 
+    @parameterized.expand(
+        [
+            # 1. The MO value is None, and the AD value is None - not a mismatch
+            (None, None, {}),
+            # 2. The MO value is "None", and the AD value is None - not a mismatch
+            ("None", None, {}),
+            # 3. The MO value is None, and the AD value is "None" - not a mismatch
+            (None, "None", {}),
+            # 4. The MO value is "None", and the AD value is "None" - not a mismatch
+            ("None", "None", {}),
+            # 5. Neither MO nor AD is None or "None" - not a mismatch
+            ("not none", "not none", {}),
+            # 6. The MO value is None, but the AD value is not None - a mismatch
+            (None, "not none", {"ad_field_name": ("not none", None)}),
+            # 7. The MO value is "None", but the AD value is not None - a mismatch
+            ("None", "not none", {"ad_field_name": ("not none", None)}),
+            # 8. The MO value is not None, but the AD value is None - a mismatch
+            ("not none", None, {"ad_field_name": (None, "not none")}),
+            # 9. The MO value is not None, but the AD value is "None" - a mismatch
+            ("not none", "None", {"ad_field_name": (None, "not none")}),
+        ]
+    )
+    def test_compare_fields_handles_none(self, mo_value, ad_value, expected_mismatch):
+        """When comparing the MO value `"None"` to the AD value `None`, do not consider
+        the AD and MO fields to differ - otherwise we will make a lot of pointless
+        updates in AD, giving empty AD fields the value `"None"`.
+
+        Even if no other AD fields need to be updated, this will cause `mo_to_ad_sync`
+        to update every AD user, causing the program to take a very long time to finish.
+
+        Also, consider the reverse situation (AD value is `"None"`, MO value is `None`)
+        as a non-mismatch.
+
+        See: #50291
+        """
+        # Arrange
+        ad_user = {"ad_field_name": ad_value}
+        ad_writer = self._prepare_adwriter()
+        # Act
+        mismatch = ad_writer._compare_fields("ad_field_name", mo_value, ad_user)
+        # Assert
+        self.assertEqual(mismatch, expected_mismatch)
+
+    @parameterized.expand(
+        [
+            (
+                [],  # manager not present in AD
+                None,  # expected `mismatch["manager"]`
+                "could not find manager by cpr 'manager_cpr'",  # expected logging
+            ),
+            (
+                # manager present in AD
+                [{"cpr_field": "manager_cpr", "DistinguishedName": "manager_dn"}],
+                ("old_manager_dn", "manager_dn"),  # expected `mismatch["manager"]`
+                "Manager should be updated",  # expected logging
+            ),
+        ]
+    )
+    def test_handles_manager_cpr_not_in_ad_dump(
+        self, ad_dump_extra, expected_manager_mismatch, expected_log_message
+    ):
+        """When `ADWriter.sync_user` is called from `run_mo_to_ad_sync`, an `ad_dump`
+        is provided. However, sometimes `sync_user` cannot find the AD user specified
+        by `mo_values["manager_cpr"] in `ad_dump` and crashes unexpectedly.
+
+        This test verifies that we no longer crash in such cases, but simply do not
+        update the `Manager` attribute in AD.
+
+        See: #50160
+        """
+        ad_writer = self._prepare_adwriter()
+        ad_dump = [
+            # employee AD user
+            self._ad_user_employee,
+            # 0 or 1 manager AD users
+            *ad_dump_extra,
+        ]
+        mo_values = {
+            "cpr": "cpr",
+            "full_name": "Full Name",
+            "manager_cpr": "manager_cpr",
+        }
+        with self.assertLogs() as actual_log_messages:
+            mismatch = ad_writer._sync_compare(mo_values, ad_dump=ad_dump)
+
+        # Assert we got the expected `mismatch["manager"]` value
+        actual_manager_mismatch = mismatch.get("manager")
+        self.assertEqual(actual_manager_mismatch, expected_manager_mismatch)
+
+        # Assert we logged the expected message
+        self.assertEqual(actual_log_messages.records[-1].message, expected_log_message)
+
+    def test_sync_compare_skips_invalid_mo_values(self):
+        ad_writer = self._prepare_adwriter(
+            # Map MO unit address to a hypothetical AD field called `ad_field_name`
+            template_to_ad_fields={"ad_field_name": "{{ mo_values['unit_city'] }}"},
+            # Ensure that MO returns an invalid unit address (DAR offline scenario)
+            read_ou_addresses={"Adresse": "Ukendt"},
+        )
+        ad_dump = [self._ad_user_employee]
+        mo_values = ad_writer.read_ad_information_from_mo("uuid")
+
+        with self.assertLogs() as actual_log_messages:
+            mismatch = ad_writer._sync_compare(mo_values, ad_dump=ad_dump)
+
+        # Assert that we did not consider the MO and AD values to be different and thus
+        # need synchronization.
+        self.assertNotIn("ad_field_name", mismatch)
+
+        # Assert that we logged the expected message
+        self.assertEqual(
+            # Find matching log message
+            only(
+                record.message
+                for record in actual_log_messages.records
+                if "INVALID" in record.message
+            ),
+            # Expected content
+            "'ad_field_name': MO value is INVALID, not changing AD value None",
+        )
+
+        # Assert that the generated PowerShell script does not include the invalid value
+        with mock.patch.object(ad_writer, "_run_ps_script") as mock_run_ps_script:
+            ad_writer.sync_user("mo_uuid", ad_dump=ad_dump)
+            ps_script = mock_run_ps_script.call_args[0][0]
+            self.assertNotIn(f'"ad_field_name"="{INVALID}"', ps_script)
+
 
 class TestPreview(_TestRealADWriter):
     def test_preview_create_command(self):
@@ -1193,3 +1333,56 @@ class TestJinjaEnv(_TestRealADWriter):
         )
         sync_cmd, _, _ = ad_writer._preview_sync_command(MO_UUID, "user_sam")
         self.assertIn(self._expected_fragment, sync_cmd)
+
+
+class TestReadADInformationFromMO(_TestRealADWriter):
+    @parameterized.expand(
+        [
+            # When DAR lookups work, MO returns addresses on the format
+            # "Testvej 123, 1234 Testby".
+            # Check that we parse such values correctly into its constituent parts.
+            (
+                {"Adresse": "Testvej 123, 1234 Testby"},
+                {"postal_code": "1234", "city": "Testby", "streetname": "Testvej 123"},
+            ),
+            # When DAR is offline, MO returns addresses with the literal value "Ukendt".
+            # We convert them into invalid unit addresses.
+            (
+                {"Adresse": "Ukendt"},
+                ADWriter.INVALID_UNIT_ADDRESS,
+            ),
+            # If we get an invalid Danish postal address, we also convert them into an
+            # invalid unit address.
+            (
+                {"Adresse": "Ikke en gyldig dansk postadresse"},
+                ADWriter.INVALID_UNIT_ADDRESS,
+            ),
+            # Empty MO response causes an invalid unit address as well.
+            (
+                {},
+                ADWriter.INVALID_UNIT_ADDRESS,
+            ),
+        ]
+    )
+    def test_parsed_addresses(self, address, expected_parsed_address):
+        ad_writers = (
+            ("ADWriter using MO API", self._get_mo_ad_writer(address)),
+            ("ADWriter using LoraCache", self._get_loracache_ad_writer(address)),
+        )
+        for name, ad_writer in ad_writers:
+            with self.subTest(name):
+                mo_values = ad_writer.read_ad_information_from_mo(MO_UUID)
+                self.assertEqual(
+                    mo_values["_parsed_addresses"], expected_parsed_address
+                )
+                if expected_parsed_address is ADWriter.INVALID_UNIT_ADDRESS:
+                    self.assertEqual(mo_values["unit_postal_code"], INVALID)
+                    self.assertEqual(mo_values["unit_city"], INVALID)
+                    self.assertEqual(mo_values["unit_streetname"], INVALID)
+
+    def _get_mo_ad_writer(self, address):
+        return self._prepare_adwriter(read_ou_addresses=address)
+
+    def _get_loracache_ad_writer(self, address):
+        lc = MockLoraCacheUnitAddress(address_value=address.get("Adresse"))
+        return self._prepare_adwriter(lc=lc, lc_historic=lc)
